=begin
*Name           : env.rb
*Description    : requires the important classes/modules for application execution
*Author         : Chandra sekaran
*Creation Date  : 24/04/2015
*Updation Date  :
=end

# Code Coverage report
# the cucumber run command should include the command line argument CODE_COVERAGE=yes (or true) in order to get code
# coverage report, which is a html report that pictorizes script LOC (Lines of COde) execution coverage
if !ENV["CODE_COVERAGE"].nil? && (["yes", "true"].include?(ENV["CODE_COVERAGE"].downcase))
  require "simplecov"
  require "simplecov-json"
  require "simplecov-rcov"
  SimpleCov.formatters = [
      SimpleCov::Formatter::RcovFormatter
  ]
  #SimpleCov::Formatter::HTMLFormatter, SimpleCov::Formatter::JSONFormatter
  SimpleCov.start
end

require "rubygems"
require "page-object"
require "watir-webdriver"
require "yaml"
require "logger"
require "fileutils"
require "time_difference"
require "data_magic"
require "require_all"
require "nokogiri"
require "base64"
require "open3"    # for capturing STDOUT, STDERR messages
require "json"     # for json file manipulation in performance report
require "dbi"      # for storing execution performance data in Sybase DB
require "appium_lib" if ENV["PLATFORM"].downcase == "mobile"  # for mobile automation

require_all "library"
require_all "object_repository"
require_all "page_objects"

World(PageObject::PageFactory)   # make PageObject available throughout the application

# environment variables moved to constants that is available throughout the application
PLATFORM = ENV["PLATFORM"] || nil     # for platform if desktop/mobile
BROWSER = ENV["BROWSER"] || nil       # for browser if firefox/chrome/internet_explorer/safari/android
BOX = ENV["BOX"] || nil               # for box name having multiple profiles
DEVICE = ENV["DEVICE"] || nil         # for mobile device if it is attached to the machine

# for resetting the profiles (in config.yml) the cucumber run command line argument should include RESET_CONFIG_VALUES=yes (or true)
# which will reset all the profiles
RESET_CONFIG_VALUES = ENV["RESET_CONFIG_VALUES"] || nil

$REPORT_FILE_NAME = "report_#{$$}"    # name of the cucumber generated report file (appended with process id as unique value)

# validating command environment variables
raise "Command Line Exception : PLATFORM can not be nil" if PLATFORM.nil?
raise "Command Line Exception : BROWSER can not be nil" if BROWSER.nil?

PageObject.javascript_framework = :jquery  # for handling AJAX

World(CUKES::PageUtils)  # make PageUtils methods global to entire TAF

World(CUKES::ObjectRepository)  # make ObjectRepository modules global to entire TAF