=begin
*Name           : hooks.rb
*Description    : hooks definition to perform task pre/post a scenario and/or step execution
*Author         : Chandra sekaran
*Creation Date  : 24/04/2015
*Updation Date  :
=end

$log = CUKES::CreateLog.new("app_env") # base log to hold the environment details

$obj_yml = CUKES::Read_From_YML.new("config/config.yml") # read the config file content

# for resetting config values
if !RESET_CONFIG_VALUES.nil?
  $obj_yml.set_value("environment/parallel_execution_count", 0)  # reset parallel execution count
  $obj_yml.release_all_profiles     # release all unused profiles
end

# set dynamic profile for a specific box or any free box
if ENV["PROFILE"].downcase == "development"
  if BOX.nil?
    BOX, PROFILE = $obj_yml.get_any_profile("development")
  else
    PROFILE = $obj_yml.get_specific_profile("development")
  end
elsif ENV["PROFILE"].downcase == "test"
  if BOX.nil?
    BOX, PROFILE = $obj_yml.get_any_profile("test")
  else
    PROFILE = $obj_yml.get_specific_profile("test")
  end
else
  raise "Invalid profile name : #{ENV["PROFILE"]}"
end

GOOGLE_URL = $obj_yml.get_value("application/#{BOX}/#{PROFILE}/url/google") # get the google url to be launched
RUBYGEMS_URL = $obj_yml.get_value("application/#{BOX}/#{PROFILE}/url/rubygems") # get the rubygems url to be launched

# Since I have used two different applications, I use two different urls, else this is not required and you can use
# the url present in config.yml file (the environment argument APP is just for demonstration only)
# For the present feature files, if you want to run rubygems feature, kindly add APP=GOOGLE in cucumber run command
# and for google feature add APP=GOOGLE in cucumber run command
if ENV["APP"].nil?
  launch_url = "#{GOOGLE_URL}"
elsif ENV["APP"].downcase.include? "google"
  launch_url = "#{GOOGLE_URL}"
elsif ENV["APP"].downcase.include? "rubygems"
  launch_url = "#{RUBYGEMS_URL}"
end

# Login credentials
USER_NAME = $obj_yml.get_value("application/#{BOX}/#{PROFILE}/login_credentials/user_name")
PASSWORD = $obj_yml.get_value("application/#{BOX}/#{PROFILE}/login_credentials/password")

# Application framework setting details
LOGGER_LEVEL = $obj_yml.get_value("environment/logger_level") # get the Logger Level
DATETIME_FORMAT = $obj_yml.get_value("environment/datetime_pattern")   # get the datetime format
FEATURE_ID_PREFIX = $obj_yml.get_value("environment/feature_id_prefix")   # get the feature id prefix
SCENARIO_ID_PREFIX = $obj_yml.get_value("environment/scenario_id_prefix")   # get the scenario id prefix

# for including Feature ID/Scenario ID columns in custom html report (report_home.html)
INCLUDE_FEATUREID_IN_REPORT = $obj_yml.get_value("environment/include_featureid_in_html_report")   # get the value whether or not to include Feature ID in html report
INCLUDE_SCENARIOID_IN_REPORT = $obj_yml.get_value("environment/include_scenarioid_in_html_report")   # get the value whether or not to include Feature ID in html report

# for consolidated html report
NO_OF_PARALLEL_THREADS = $obj_yml.get_value("environment/no_of_parallel_threads") # get the parallel threads count
DELAY_BETWEEN_PARALLEL_THREADS = $obj_yml.get_value("environment/delay_between_parallel_threads") # get the delay between parallel threads

# Performance report
# the cucumber execution details are extracted from report(s) generated and saved into Sybase database
# below are the database details
PERFORMANCE_REPORT = $obj_yml.get_value("environment/performance_report")
if ["yes", "true"].include?(PERFORMANCE_REPORT.to_s.downcase)
  DB_SERVER = $obj_yml.get_value("environment/db_server")
  DB_NAME = $obj_yml.get_value("environment/db_name")
  DB_USER_NAME = $obj_yml.get_value("environment/db_user_name")
  DB_PASSWORD = $obj_yml.get_value("environment/db_password")
end

# log test execution environment details
$log.info("__________________________________________________________")
$log.info("Test Machine         : #{ENV["COMPUTERNAME"]}(#{ENV['OS']})")
$log.info("Test Browser         : #{BROWSER}")
$log.info("Test URL             : #{launch_url}")   # the actual url under test can be given here
$log.info("__________________________________________________________")

$current_log_dir = $log.get_current_log_dir   # global variable to hold the base log directory name

$start_time = $log.get_current_datetime     # start time of the execution

$log_env = $log       # to hold base log file object for log entries after test execution

$current_log_file = nil       # hold the current log file name

PageObject.default_element_wait = $obj_yml.get_value("environment/default_element_wait")  # set default timeout for element wait
PageObject.default_page_wait = $obj_yml.get_value("environment/default_page_wait")  # set default timeout for page wait

$browser = CUKES::BrowserSettings.browser_setup(BROWSER)     # launches the browser

# if you want to launch the URL only once (at the beginning), then you can use the below script
$browser.navigate.to(launch_url)

$scenario_count = 0    # holds scenario count for each feature file

# Description       : called before the execution of a scenario
# Author            : Chandra sekaran
# Arguments         :
#   scenario        : scenario object
#
Before do |scenario|
  @step_count = nil  # step counter used in AfterStep to get current step name

  @scenario_start_time = $log.get_current_datetime     # scenario execution start time

  @browser = $browser    # passes the browser object to the page class constructor (implicitly)
  @browser.manage.timeouts.implicit_wait = 3  # set 3s implicit wait time
  $browser_version = @browser.capabilities[:version]    # current browser version

  $world = self          # for overriding puts method to print the argument into html file and in console as well 

  $str_feature_file_path = scenario.file       # absolute path of current feature file

  @str_feature_module_name = $log.get_feature_module_name($str_feature_file_path)   # extracts module (and/or submodule) name from str_file_path
  if $current_log_file.nil? || !($current_log_file.include? @str_feature_module_name)
    $log = CUKES::CreateLog.new(@str_feature_module_name)       # creates a new log file with the module ame
    $current_log_file = $log.get_current_log_file
  end

  $log.info("__________________________________________________________")

  # feature name
  case scenario
    when Cucumber::Ast::Scenario
      @feature_name = scenario.feature.name
    when Cucumber::Ast::OutlineTable::ExampleRow
      @feature_name = scenario.scenario_outline.feature.name
  end
  $log.info("Test Feature         : " + @feature_name)

  # check for a new feature and set $scenario_count accordingly
  if $feature_name_old != @feature_name
    $scenario_count = 0
    $feature_name_old = @feature_name
  end

  # scenario name
  case scenario
    when Cucumber::Ast::Scenario
      @scenario_name = scenario.name
    when Cucumber::Ast::OutlineTable::ExampleRow
      @scenario_name = scenario.scenario_outline.name
  end
  $log.info("Test Scenario        : " + @scenario_name)
  Kernel.puts("\n [#{Time.now.strftime(DATETIME_FORMAT)}] Currently running scenario : \n #{@scenario_name.to_s}")   # for logging in console

  # tags name
  $scenario_tags = scenario.source_tag_names
  $log.info("Test tag(s)          : " + $scenario_tags.to_s)
  Kernel.puts("\n \tWith tag(s) : #{$scenario_tags.to_s}")   # for logging in console

  # get first step name of first scenario
  @current_feature = if scenario.respond_to?('scenario_outline')
                       # execute the following code only for scenarios outline (starting from the second example)
                       scenario.scenario_outline.feature
                     else
                       # execute the following code only for a scenario and a scenario outline (the first example only)
                       scenario.feature
                     end
  $log.info("__________________________________________________________")
  @arr_steps = []
  @arr_steps = get_steps(@current_feature)  # get all steps under the current scenario
end

# Description       : called after the execution of a step
# Author            : Chandra sekaran
# Arguments         :
#   scenario        : scenario object
#
AfterStep do |scenario|
  sleep 1
  @step_count = 0 if @step_count.nil?   # @step_count ||= 0

  begin
    $log.info("Test Step (#{scenario.failed? ? 'failed' : 'success'})  : " + @arr_steps[@step_count])
  rescue Exception => e
    $log.error("Error in AfterStep hook for step_count (#{@step_count}): #{e}")
    $log.info("Test Step (#{scenario.failed? ? 'failed' : 'success'})  : " + @arr_steps[@step_count-1]) rescue Exception
  end
  Kernel.puts("\n \t\t[#{Time.now.strftime(DATETIME_FORMAT)}] #{@arr_steps[@step_count].to_s}") rescue Exception  # for logging in console

  @step_count += 1       # increase step counter
end

# Description       : called after support has been loaded but before features are loaded
# Author            : Chandra sekaran
# Arguments         :
#   config          : config object
#
AfterConfiguration do |config|

end

# Description       : called after the execution of a scenario
# Author            : Chandra sekaran
# Arguments         :
#   scenario        : scenario object
#
After do |scenario|
  @step_count = 0 if @step_count.nil?
  @scenario_end_time = $log.get_current_datetime    # scenario execution finish time

  if scenario.failed?
    begin
      $log.info("Test Step (failed)   : " + @arr_steps[@step_count])
    rescue Exception => e
      $log.error("Error in After hook for step_count (#{@step_count}): #{e}")
      $log.info("Test Step (failed)   : " + @arr_steps[@step_count-1]) rescue Exception
    end
    str_img_path = CUKES::BrowserSettings.capture_screenshot(@str_feature_module_name)   # takes the screenshot of webpage

    # attaches a link to html page, on click of which shows the image in the web page
    embed(str_img_path, "image/png", "Click to view screenshot") rescue Exception
  end

  $scenario_count += 1    # increment scenario count by 1

  $log.info("__________________________________________________________")
  $log.info("Scenario start time  : " + $log.get_formatted_datetime(@scenario_start_time))
  $log.info("Scenario end time    : " + $log.get_formatted_datetime(@scenario_end_time))
  $log.info("Total elapsed time   : " + $log.get_datetime_diff(@scenario_start_time, @scenario_end_time))
  $log.info("__________________________________________________________")
end

# Description       : called after the execution of all features
# Author            : Chandra sekaran
#
at_exit do
  $obj_yml.set_value("application/#{BOX}/#{PROFILE}/in_use", "no")   # releases the current profile
  $log_env.info("Releasing the currently used profile '#{PROFILE}' under '#{BOX}' box")

  $obj_yml.change_execution_count("environment/parallel_execution_count", $obj_yml.get_value("application/#{BOX}/#{PROFILE}/in_use"))

  $end_time = $log_env.get_current_datetime

  # logs into the base log file created with the name 'app_env.log'
  $log_env.info("__________________________________________________________")
  $log_env.info("Execution start time : " + $log_env.get_formatted_datetime($start_time))
  $log_env.info("Execution end time   : " + $log_env.get_formatted_datetime($end_time))
  $log_env.info("Total elapsed time   : " + $log_env.get_datetime_diff($start_time, $end_time).to_s)
  $log_env.info("__________________________________________________________")

  # rename the html report file and move it to respective log report directory
  $log_env.create_html_report

  # creates a new custom HTML report based on cucumber report(s) generated only after complete (single or parallel) execution
  if $obj_yml.get_value("environment/parallel_execution_count").to_i == 0
    # get the html report files of current execution
    arr_report_file = $log_env.get_files_absolute_path("test_result", "html", $start_time - $world.get_execution_delay_time.seconds)
    $log.info("Report file names (html): #{arr_report_file.to_s}")

    obj = CUKES::CustomHtmlReport.new(arr_report_file)
    obj.create_custom_report

    if ["yes", "true"].include?(PERFORMANCE_REPORT.to_s.downcase)
      # get the json report files of current execution
      arr_report_file = $log_env.get_files_absolute_path("test_result", "json", $start_time - $world.get_execution_delay_time.seconds)
      $log.info("Report directory names (json): #{arr_report_file.to_s}")

      obj = CUKES::PerformanceReport.new(arr_report_file)
      obj.create_performance_report
    end
  end
  CUKES::BrowserSettings.quit_browser   # closes the current browser
end